/*--------------------------------------------------------------------*/
/* conditions.c                                                       */
/* Author: Bob Dondero                                                */
/* To build use the -pthread option                                   */
/*--------------------------------------------------------------------*/

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>

/*--------------------------------------------------------------------*/

/* Think of this as a BankAcct object. */

static int bankAcctBalance = 0;
static pthread_mutex_t bankAcctMutex;
static pthread_cond_t bankAcctCondition;

static void deposit(int amount)
{
   pthread_mutex_lock(&bankAcctMutex);
   bankAcctBalance += amount;
   printf("%d\n", bankAcctBalance);
   pthread_cond_signal(&bankAcctCondition);
   pthread_mutex_unlock(&bankAcctMutex);
}

static void withdraw(int amount)
{
   pthread_mutex_lock(&bankAcctMutex);
   while (bankAcctBalance < amount)
      pthread_cond_wait(&bankAcctCondition, &bankAcctMutex);
   bankAcctBalance -= amount;
   printf("%d\n", bankAcctBalance);
   pthread_mutex_unlock(&bankAcctMutex);
}

/*--------------------------------------------------------------------*/

static void *makeDeposits()
{
   int i;
   for (i = 0; i < 10; i++)
      deposit(1);
   return NULL;
}

/*--------------------------------------------------------------------*/

static void *makeWithdrawals()
{
   int i;
   for (i = 0; i < 5; i++)
      withdraw(2);
   return NULL;
}

/*--------------------------------------------------------------------*/

int main(void)
{
   pthread_t depositThread;
   pthread_t withdrawThread;

   pthread_mutex_init(&bankAcctMutex, NULL);
   pthread_cond_init(&bankAcctCondition, NULL);

   pthread_create(&depositThread, NULL, makeDeposits, NULL);
   pthread_create(&withdrawThread, NULL, makeWithdrawals, NULL);

   pthread_join(depositThread, NULL);
   pthread_join(withdrawThread, NULL);

   pthread_mutex_destroy(&bankAcctMutex);
   pthread_cond_destroy(&bankAcctCondition);

   printf("Final bankAcctBalance: %d\n", bankAcctBalance);
   return 0;
}
