#!/usr/bin/env python

#-----------------------------------------------------------------------
# daytimeserver.py
# Author: Bob Dondero
#-----------------------------------------------------------------------

import os
import sys
import socket
import time
import multiprocessing

#-----------------------------------------------------------------------

CDELAY = int(os.environ.get('CDELAY', '0'))

#-----------------------------------------------------------------------

def consume_cpu_time(delay):

    initial_thread_time = time.thread_time()
    while (time.thread_time() - initial_thread_time) < delay:
        pass

#-----------------------------------------------------------------------

def handle_client(sock):

    print('Forked child process')

    try:

        # Simulate a compute-bound server.
        consume_cpu_time(CDELAY)

        datetime = time.asctime(time.localtime())
        flo = sock.makefile(mode='w', encoding='ascii')
        flo.write(datetime + '\n')
        flo.flush()

        sock.close()
        print('Closed socket in child process')
        print('Exiting child process')

    except Exception as ex:
        print(ex, file=sys.stderr)
        sys.exit(1)

#-----------------------------------------------------------------------

def main():

    if len(sys.argv) != 2:
        print('Usage: python %s port' % sys.argv[0])
        sys.exit(1)

    try:
        port = int(sys.argv[1])

        server_sock = socket.socket()
        print('Opened server socket')
        if os.name != 'nt':
            server_sock.setsockopt(
                socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server_sock.bind(('', port))
        print('Bound server socket to port')
        server_sock.listen()
        print('Listening')

        while True:
            sock, _ = server_sock.accept()
            with sock:
                print('Accepted connection')
                print('Opened socket')
                process = multiprocessing.Process(
                    target=handle_client,
                    args=[sock])
                process.start()
            print('Closed socket in parent process')

    except Exception as ex:
        print(ex, file=sys.stderr)
        sys.exit(1)

#-----------------------------------------------------------------------

if __name__ == '__main__':
    main()
